/*
 * Copyright 1993-97 by Apple Computer, Inc.
 *              All Rights Reserved 
 *  
 * Permission to use, copy, modify, and distribute this software and 
 * its documentation for any purpose and without fee is hereby granted, 
 * provided that the above copyright notice appears in all copies and 
 * that both the copyright notice and this permission notice appear in 
 * supporting documentation. 
 *  
 * APPLE COMPUTER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE 
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE. 
 *  
 * IN NO EVENT SHALL APPLE COMPUTER BE LIABLE FOR ANY SPECIAL, INDIRECT, OR 
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM 
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT, 
 * NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION 
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
 */
#include "SCSIStuff.h"
#include "io.h"

int
DoRead(
	media *		    dev,
	UInt32              block,
	UInt16              count,
	char *              addr
    )
{
    OSErr                   status;
    Str255                  errorText;
    char*       msg;
    static SCSI_10_Byte_Command gReadCommand = {
	kScsiCmdRead10, 0, 0, 0, 0, 0, 0, 0, 0, 0
    };
    SCSI_Sense_Data         senseData;
    DeviceIdent scsiDevice;
    int rtn_value;

    scsiDevice.diReserved = 0;
    if (dev->fd2 < 0) {
	scsiDevice.bus = kOriginalSCSIBusAdaptor;
    } else {
	scsiDevice.bus = dev->fd2;
    }
    scsiDevice.targetID = dev->fd;
    scsiDevice.LUN = 0;

    gReadCommand.lbn4 = (block >> 24) & 0xFF;
    gReadCommand.lbn3 = (block >> 16) & 0xFF;
    gReadCommand.lbn2 = (block >> 8) & 0xFF;
    gReadCommand.lbn1 = block & 0xFF;

    gReadCommand.len2 = (count >> 8) & 0xFF;
    gReadCommand.len1 = count & 0xFF;

    status = DoSCSICommand(
			    scsiDevice,
			    "\pRead",
			    (SCSI_CommandPtr) &gReadCommand,
			    (Ptr) addr,
			    count * dev->block_size,
			    scsiDirectionIn,
			    NULL,
			    &senseData,
			    errorText
	    );
    if (status == noErr) {
	rtn_value = 1;
    } else {
	rtn_value = 0;
    }
    return rtn_value;
}

int
DoWrite(
	media *		    dev,
	UInt32              block,
	UInt16              count,
	char *              addr
    )
{
    OSErr                   status;
    Str255                  errorText;
    char*       msg;
    static SCSI_10_Byte_Command gWriteCommand = {
	kScsiCmdWrite10, 0, 0, 0, 0, 0, 0, 0, 0, 0
    };
    SCSI_Sense_Data         senseData;
    DeviceIdent scsiDevice;
    int rtn_value;

    scsiDevice.diReserved = 0;
    if (dev->fd2 < 0) {
	scsiDevice.bus = kOriginalSCSIBusAdaptor;
    } else {
	scsiDevice.bus = dev->fd2;
    }
    scsiDevice.targetID = dev->fd;
    scsiDevice.LUN = 0;

    gWriteCommand.lbn4 = (block >> 24) & 0xFF;
    gWriteCommand.lbn3 = (block >> 16) & 0xFF;
    gWriteCommand.lbn2 = (block >> 8) & 0xFF;
    gWriteCommand.lbn1 = block & 0xFF;

    gWriteCommand.len2 = (count >> 8) & 0xFF;
    gWriteCommand.len1 = count & 0xFF;

    status = DoSCSICommand(
			    scsiDevice,
			    "\pWrite",
			    (SCSI_CommandPtr) &gWriteCommand,
			    (Ptr) addr,
			    count * dev->block_size,
			    scsiDirectionOut,
			    NULL,
			    &senseData,
			    errorText
	    );
    if (status == noErr) {
	rtn_value = 1;
    } else {
	rtn_value = 0;
    }
    return rtn_value;
}


int
DoReadCapacity(
	media *		    dev,
	UInt32		    *blockCount,
	UInt32		    *blockSize
	)
{
    OSErr		status;
    Str255		errorText;
    static const SCSI_10_Byte_Command gCapacityCommand = {
	kScsiCmdReadCapacity, 0, 0, 0, 0, 0, 0, 0, 0, 0
    };
    SCSI_Sense_Data	senseData;
    DeviceIdent		scsiDevice;
    SCSI_Capacity_Data  capacityData;
    UInt32		temp;
    int rtn_value;

    scsiDevice.diReserved = 0;
    if (dev->fd2 < 0) {
	scsiDevice.bus = kOriginalSCSIBusAdaptor;
    } else {
	scsiDevice.bus = dev->fd2;
    }
    scsiDevice.targetID = dev->fd;
    scsiDevice.LUN = 0;

    CLEAR(capacityData);

    status = DoSCSICommand(
	scsiDevice,
	"\pRead Capacity",
	(SCSI_CommandPtr) &gCapacityCommand,
	(Ptr) &capacityData,
	sizeof (SCSI_Capacity_Data),
	scsiDirectionIn,
	NULL,
	&senseData,
	errorText
    );

    if (status == noErr) {
	temp = capacityData.lbn4;
	temp = (temp << 8) | capacityData.lbn3;
	temp = (temp << 8) | capacityData.lbn2;
	temp = (temp << 8) | capacityData.lbn1;
	*blockCount = temp;

	temp = capacityData.len4;
	temp = (temp << 8) | capacityData.len3;
	temp = (temp << 8) | capacityData.len2;
	temp = (temp << 8) | capacityData.len1;
	*blockSize = temp;

	rtn_value = 1;
    } else {
	rtn_value = 0;
    }
    return rtn_value;
}


